@@ -1,6 +1,6 @@
 package TPath;
 {
-  $TPath::VERSION = '0.010';
+  $TPath::VERSION = '0.011';
 }
 
 # ABSTRACT: general purpose path languages for trees
@@ -17,7 +17,7 @@ TPath - general purpose path languages for trees
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 SYNOPSIS
 
@@ -138,7 +138,7 @@ something that understands your trees. In general to use C<TPath> you instantiat
 then call the forester's methods.
 
 Forester objects make use of an index (L<TPath::Index>), which caches information not present in, or
-not cheaply from, the nodes themselves. If no index is explicitly provided it is created, but one
+not cheaply extracted from, the nodes themselves. If no index is explicitly provided it is created, but one
 can gain some efficiency by reusing an index when select paths from a tree. One can use a forester's
 C<index> method to produce a C<TPath::Index>.
 
@@ -293,6 +293,12 @@ backslash. Any unexpected character must be escaped. So
 
 represents the literal C<a\b>.
 
+There is also a quoting convention that one can use to avoid many escapes inside a tag name.
+
+  /:"a tag name you otherwise would have to put a lot of escapes in"
+
+See the Grammar section below for details.
+
 =head3 ~a~ regex
 
 =over 2
@@ -312,7 +318,7 @@ Any attribute may be used as a selector so long as it is preceded by something o
 the null separator -- in other words, C<@> cannot be the first character in a path. This is because 
 attributes may take arguments and among other things these arguments can be both expressions and 
 other attributes. If C<@foo> were a legitimate path expression it would be ambiguous how to compile 
-C<@bar(@foo)>. Is the argument an attribute or a path with an attribute selector. You can produce
+C<@bar(@foo)>. Is the argument an attribute or a path with an attribute selector? You can produce
 the effect of an attribute selector with the null separator, however, in two ways
 
 =over 2
@@ -326,6 +332,9 @@ the effect of an attribute selector with the null separator, however, in two way
 the second of these will be normalized in parsing to precisely what one would expect with a C<@foo>
 path.
 
+The attribute naming conventions are the same as those of tags with the exception that attributes are
+always preceded by C<@>.
+
 =head3 complement selectors
 
 The C<^> character before a literal, regex, or attribute selector will convert it into an attribute selector.
@@ -780,9 +789,10 @@ Attribute parameters are enclosed within parentheses. Within these parentheses,
 delimited by commas. Space is optional around parameters.
 
 For the standard attribute set available to all expressions, see L<TPath::Attributes::Standard>.
-For the extended set that can be composed in, see L<TPath::Attributes::Extended>. There are
-various ways one can add bespoke attributes but the easiest is to add them to an individual
-forester via the C<add_attribute> method:
+For the extended set that can be composed in, see L<TPath::Attributes::Extended>.
+
+There are various ways one can add bespoke attributes but the easiest is to add them to an 
+individual forester via the C<add_attribute> method:
 
   my $forester = MyForester->new;
   $forester->add_attribute( 'foo' => sub {
@@ -917,6 +927,169 @@ This is verbose, but then this is not likely to be a common requirement.
 The TPath semantics facilitate the implementation of repetition, which is absent from
 XPath.
 
+=head2 Grammar
+
+The following is a BNf-style grammar of the TPath expression language. It is the actual parsing code,
+in the L<Regexp::Grammars> formalism, used to parse expressions minus the bits that improve efficiency
+and adjust the construction of the abstract syntax tree produced by the parser.
+
+    ^ <treepath> $
+    
+       <rule: treepath> <[path]> ( \| <[path]> )*
+    
+       <token: path> (?!@) <segment>+
+    
+       <token: segment> <separator>? <step> | <cs>
+       
+       <token: quantifier> [?+*] | <enum>
+       
+       <rule: enum> [{] \d*+ ( , \d*+ )? [}]
+       
+       <token: grouped_step> \( \s*+ <treepath> \s*+ \) <quantifier>?
+    
+       <token: id>
+          :id\( ( (?>[^\)\\]|\\.)++ ) \)
+    
+       <token: cs>
+          <separator>? <step> <quantifier>
+          | <grouped_step>
+    
+       <token: separator> \/[\/>]?+
+    
+       <token: step> <full> <[predicate]>* | <abbreviated>
+           
+       <token: full> <axis>? <forward>
+    
+       <token: axis> (?<!//) (?<!/>) (<%AXES>) ::
+    
+       <token: abbreviated> (?<!/[/>]) ( \.{1,2}+ | <id> | :root )
+    
+       <token: forward> <wildcard> | ^? ( <specific> | <pattern> | <attribute> )
+    
+       <token: wildcard> \* <start_of_path>
+       
+       <token: start_of_path> # somewhat lame way to make sure * quantifier isn't misinterpreted as the wildcard character
+          (?<=[/:>].)
+          | (?<=\(.)
+          | (?<=\(\s.)
+          | (?<=\(\s{2}.)
+          | (?<=\(\s{3}.)
+          | (?<=\(\s{4}.) # if the user puts more than 4 whitespace characters between ) and *, it will be mis-parsed
+          | (?<=\A.)
+          | (?<=\A\s.)
+          | (?<=\A\s{2}.)
+          | (?<=\A\s{3}.)
+          | (?<=\A\s{4}.)
+    
+       <token: specific>
+          <name>
+    
+       <token: pattern>
+          (~(?>[^~]|~~)++~)
+    
+       <token: aname>
+          @ <name>
+       
+       <token: name>
+          (\\.|[\p{L}\$_])(?>[\p{L}\$\p{N}_]|[-.:](?=[\p{L}_\$\p{N}])|\\.)*+
+          | <qname>
+       
+       <token: qname> 
+          : (\p{PosixPunct}.+?\p{PosixPunct}) 
+          <require: (?{qname_test($^N)})> 
+     
+       <rule: attribute> <aname> <args>?
+    
+       <rule: args> \( <arg> ( , <arg> )* \)
+    
+       <token: arg>
+          <treepath> | <literal> | <num> | <attribute> | <attribute_test> | <condition>
+    
+       <token: num> <signed_int> | <float>
+    
+       <token: signed_int> [+-]?+ <int>   
+    
+       <token: float> [+-]?+ <int>? \.\d++ ( [Ee][+-]?+ <int> )?+
+    
+       <token: literal>
+          <squote> | <dquote>
+    
+       <token: squote> ' ([^'\\]|\\.)*+ '
+    
+       <token: dquote> " ([^"\\]|\\.)*+ "   
+    
+       <rule: predicate>
+          \[ ( <signed_int> | <condition> ) \]
+    
+       <token: int> \b(?:0|[1-9][0-9]*+)\b
+    
+       <rule: condition> 
+          <not>? <item> ( <operator> <not>? <item> )*
+
+       <token: not>
+             ( ! | (?<=[\s\[(]) not (?=\s) ) 
+             ( \s*+ (?: ! | (?<=\s) not (?=\s) ) )*+ 
+       
+       <token: operator>
+          ( <or> | <xor> | <and> )
+       
+       <token: xor>
+          ( ` | (?<=\s) one (?=\s) )
+           
+       <token: and>
+          ( & | (?<=\s) and (?=\s) )
+           
+       <token: or>
+          ( \|{2} | (?<=\s) or (?=\s) )
+    
+       <token: term> 
+          <attribute> | <attribute_test> | <treepath>
+    
+       <rule: attribute_test>
+          <attribute> <cmp> <value> | <value> <cmp> <attribute>
+    
+       <token: cmp> [<>=]=?+|![=~]|=~
+    
+       <token: value> <literal> | <num> | <attribute>
+    
+       <rule: group> \( <condition> \)
+    
+       <token: item>
+          <term> | <group>
+
+The crucial part, most likely, is the definition of the <name> rule which governs what you can put in
+tags and attribute names without escaping. The rule, copied from above, is
+
+          (\\.|[\p{L}\$_])(?>[\p{L}\$\p{N}_]|[-.:](?=[\p{L}_\$\p{N}])|\\.)*+
+          | <qname>
+
+This means a tag or attribute name begins with a letter, the dollar sign, or an underscore, and is followed by
+these characters or numbers, or dashes, dots, or colons followed by these characters. And at any time one can
+violate this basic rule by escaping a character that would put one in violation with the backslash character, which
+thus cannot itself appear except when escaped.
+
+Alternatively, one can "quote" the entire expression following the C<qname> convention:
+
+          : (\p{PosixPunct}.+?\p{PosixPunct}) 
+          <require: (?{qname_test($^N)})> 
+
+A quoted name begins with a colon followed by some delimiter character, which must be a POSIX punctuation mark. These
+are the symbols
+
+  [!"#$%&'()*+,\-./:;<=>?@[\\\]^_`{|}~]
+
+B<Note that brackets are not expected to be balanced!> The entire delimited name is bracketed by a pair of these delimiter
+characters. Within these delimiters any characters may occur. The delimiters themselves, and \, may occur if they are
+escaped in the usual way. Since the C<qname> convention commits you to 3 extra-name characters before any escapes, it
+is generally not advisable unless you otherwise would have to escape more than 3 characters or you feel that whatever
+escaping you would have to do would mar legibility. Double and single quotes make particularly legible C<qname> delimiters
+if it comes to that. Compare
+
+  /home/bob//file\ name\ with\ spaces
+  /home/bob//:"file name with spaces"
+
+One uses the same number of characters in each case but the second is clearly easier on the eye.
+
 =head1 HISTORY
 
 I wrote TPath initially in Java (L<http://dfhoughton.org/treepath/>) because I wanted a more 
