@@ -8,11 +8,6 @@ sub attr : Attr {
     $n->attribute($name);
 }
 
-sub tag : Attr {
-    my ( $self, $n, $i, $c, $name ) = @_;
-    $n->tag;
-}
-
 sub te : Attr {
     my ( $self, $n, $i, $c, $name ) = @_;
     $n->tag eq $name ? 1 : undef;
@@ -21,14 +16,14 @@ sub te : Attr {
 package ToyXMLForester;
 
 use Moose;
+use MooseX::MethodAttributes;
 use namespace::autoclean;
 use TPath::Index;
 
 with qw(TPath::Forester MyAttributes);
 
 sub children { my ( $self, $n ) = @_; $n->children }
-sub has_tag     { my ( $self, $n, $tag ) = @_; $n->tag eq $tag }
-sub matches_tag { my ( $self, $n, $rx )  = @_; $n->tag =~ $rx }
+sub tag : Attr    { my ( $self, $n ) = @_; $n->tag }
 sub id { my ( $self, $n ) = @_; $n->attribute('id') }
 
 sub BUILD { $_[0]->_node_type('Element') }
